
import 'dart:async';
import 'dart:collection';
import 'dart:convert';

import 'package:flutter/services.dart';

class LgcreException
{
  String _message = "";

  LgcreException([String message = 'Invalid value']) {
    this._message = message;
  }

  @override
  String toString() {
    return _message;
  }
}

class ResultEntry
{
  ResultEntry(this.poi, this.score);

  /// The string id of the poi
  final String poi;

  /// The recommendation score
  final double score;

  static ResultEntry fromJson(String str) {
    Map<String,dynamic> obj = json.decode(str);
    return ResultEntry(obj['poi'] as String, obj['score'] as double);
  }
}

class Lgcre {

  static const MethodChannel _channel =
  const MethodChannel('lgcre');

  /// Initializes LGCRE with the given [options]
  /// 
  /// Available [options] are :
  /// 
  ///     ['lgcre.licenseid']: the license id
  ///
  ///     ['lgcre.clientid']: the client id
  ///
  ///     ['lgcre.loglevel']: the log level (off, fine, warning, debug, severe, config),
  ///
  ///     ['lgcre.maxmem']: the maximum amount of memory to use (affects the speed)
  ///
  /// The methods throws a [LgcreException] if initialization fails. This method must be
  /// be called synchronously with await or must be guaranteed that its execution is finished
  /// before calling any other method
  static Future<void> init(Map<String,String> options) async
  {
    try
    {
      await _channel.invokeMethod<void>('init', options);
    } on PlatformException catch (e)
    {
      throw new LgcreException('${e.message}');
    }
  }

  /// Resets the stored data
  ///
  /// The methods throws a [LgcreException] if resetting data fails. This method is transactional
  /// and can be executed concurrently with other transactional methods
  static Future<void> reset() async
  {
    try
    {
      await _channel.invokeMethod('reset');
    } on PlatformException catch (e)
    {
      throw new LgcreException('${e.message}');
    }

  }

  /// Adds [data] to the recommender engine
  ///
  /// The [data] map can contain has the following fields
  ///
  ///     ['pois'] : a list of strings identifying the pois
  ///
  ///     ['categories'] : a list of strings identifying the pois
  ///
  ///     ['links_categories'] : a list of maps with:
  ///
  ///           ['category1']: The string id of the first category to link
  ///
  ///           ['category2']: The string id of the second category to link
  ///
  ///     ['links_poi_category']: a list of maps with:
  ///
  ///           ['poi']: the string id of the poi to link
  ///
  ///           ['category']: the stdring id of the category to link
  ///
  ///     ['person']: A map with:
  ///
  ///       ['pois']: a list of string ids of pois a person like/visited
  ///
  ///       ['categories']: a list of string ids of categories the person is interested in 
  ///
  /// The methods throws a [LgcreException] if setting data fails, for instance:
  /// 1) if passing an invalid key in the data map, 2) adding a link between
  /// two unexisting entities that are not bein added in this call or have been
  /// added in a previous call to add data or 3) an internal error occurs
  ///
  /// Entities (categories, pois etc.) are identified by unique ids. If an
  /// entity is added twice, either in the same call or in different calls, only
  /// one time is added. The same happens for relations/links between items added
  /// by means of the links_categories, links_poi_category, person poi's and person categories's fields. Such 
  /// relations/links are only added once.
  ///
  /// This method is transactional and can be executed concurrently with other transactional methods
  static Future<void> addData(Map<String, dynamic> data)  async
  {
    try
    {
      await _channel.invokeMethod('addData', data);

    } on PlatformException catch (e)
    {
      throw new LgcreException('${e.message}');
    }
  }

  /// Removes [data] from the recommender engine
  ///
  /// The [data] map can contain has the following fields
  ///
  ///     ['pois'] : a list of strings identifying the pois
  ///
  ///     ['categories'] : a list of strings identifying the pois
  ///
  ///     ['links_categories'] : a list of maps with:
  ///
  ///           ['category1']: The string id of the first category to link
  ///
  ///           ['category2']: The string id of the second category to link
  ///
  ///     ['links_poi_category']: a list of maps with:
  ///
  ///           ['poi']: the string id of the poi to link
  ///
  ///           ['category']: the stdring id of the category to link
  ///
  ///     ['person']: A map with:
  ///
  ///       ['pois']: a list of string ids of pois a person like/visited
  ///
  ///       ['categories']: a list of string ids of categories the person is interested in 
  ///
  /// The methods throws a [LgcreException] if setting data fails, for instance:
  /// 1) if passing an invalid key in the data map, 2) removing a link between
  /// two unexisting entities or 3) an internal error occurs
  ///
  /// Entities (categories, pois etc.) are identified by unique ids. If an
  /// unexising entity is removed twice nothing happens. The same happens for
  /// relations/links between entities added, it the link does not exist,
  /// nothing happens.
  ///
  /// If both entities and links between those entities are removed in the same
  /// call, it is guaranteed that links are removed first and entities second to
  /// prevent throwing an error for an entity not existing when removing the link
  ///
  /// This method is transactional and can be executed concurrently with other transactional methods
  static Future<void> removeData(Map<String, dynamic> data)  async
  {
    try
    {
      await _channel.invokeMethod('removeData', data);

    } on PlatformException catch (e)
    {
      throw new LgcreException('${e.message}');
    }
  }


  /// Releases the resources used by the library. This method cannot be called while other
  /// methods such as run, addData, removeData or reset are being called
  static Future<void> release() async
  {
    try
    {
      await _channel.invokeMethod('release');
    } on PlatformException catch (e)
    {
      throw new LgcreException('${e.message}');
    }
  }

  /// Runs a recommendation [query]
  ///
  /// The [query] is a map with:
  ///
  ///     ['pois']: a list of maps with
  ///         ['poi']: an string with the poi identifier
  ///         ['categories']: a list of string ids with the categories of the poi
  ///
  /// The query returns a list of [ResultEntry] sorted decreasingly by the
  /// [ResultEntry] score.
  /// The methods throws a [LgcreException] if an internal error occurs and the query cannot be executed.
  /// This method is transactional and can be executed concurrently with other transactional methods
  static Future<List<ResultEntry>?> run(Map<String,dynamic> query) async
  {
    try
    {
      final List<String>? entries = await _channel.invokeListMethod<String>('run', query);
      return entries?.map(ResultEntry.fromJson).toList() ?? <ResultEntry>[];
    } on PlatformException catch (e)
    {
      throw new LgcreException('${e.message}');
    }
  }
}
